// #define GFSTRINGSTUFF 1
/* Uncomment one of these 3 lines. */
#define GFSYMBOLSIZE 8
// #define GFSYMBOLSIZE 16
// #define GFSYMBOLSIZE 32

#if GFSYMBOLSIZE==8
#define GFSYMBOL unsigned char
#elif GFSYMBOLSIZE==16
#define GFSYMBOL unsigned short
#elif GFSYMBOLSIZE==32
#define GFSYMBOL unsigned long
#else
#error GFSYMBOLSIZE must be 8, 16, or 32
#endif
typedef struct {
  int  order;       /* Number of bits in a symbol. */
  int  size;        /* Size of the field (2**order). Code length is size-1. */
  GFSYMBOL *atable; /* table[i] = alpha**i. Length size-1. */
  GFSYMBOL *amap;   /* map[alpha**i] = i. Length size-1.   */
  unsigned long minpoly; /* The generator polynomial. */
} GaloisField2
;
 /* Return primitive polynomial for 2**m, or 0 for invalid m. */
GFSYMBOL GaloisPrimPoly(
int m  /* 3 <= m <= NumGFprimPolys */
)
;
/*Usage: GaloisField2 GF; ok=GaloisFieldAllocate(&GF,order); **
** Returns 0 on error. Requires malloc.                      */

int GaloisFieldAllocate(GaloisField2 *GF2,int order)
;
/*Usage: GaliosField2 *GF2 = GaloisFieldCreate(order); **
** Returns (void *)0 on error. Requires malloc.        */

GaloisField2 *GaloisFieldCreate(int order)
;
/* Call after GaloisFieldAllocate or GaloisFieldCreate.
** Builds tables to convert between Power and Polynomial representations.
** See L&C table 2.8 on p. 33
** Errors: None (but expect a program crash if GF isn't set up yet.)
*/
void GaloisFieldBuild(     /* Table generation (L&C-33) */
  GaloisField2 *GF        /* The computed field parameters and tables. */
)
;
/* Call this when done, only if GaloisFieldAllocate was used. */
void GaloisFieldDeallocate(  /* Deallocate the buffers. */
  GaloisField2 *GF
)
;
/* Call this when done, only if GaloisFieldCreate was used. */
void GaloisFieldDestroy(  /* Deallocate the GF buffers and the structure. */
  GaloisField2 *GF
)
;
 GFSYMBOL GaloisMpy(   /* Return a*b */
  GaloisField2 *GF,
  GFSYMBOL a,
  GFSYMBOL b
)
;
 GFSYMBOL GaloisRecip( /* Return 1/b, or 0 if b==0. */
  GaloisField2 *GF,
  GFSYMBOL b
)
;
 GFSYMBOL GaloisDiv( /* Return a/b, or 0 if b==0. */
  GaloisField2 *GF,
  GFSYMBOL a,
  GFSYMBOL b
)
;
 char *GaloisToString( /* Return value as character string. */
  GaloisField2 *GF,
  GFSYMBOL val,    /* The input value in n-tuple form. */
  int fmttype,     /* Input: Output form and optional format flag:
		   ** 0 - Power of alpha: 0 or @n
		   ** 1 - Bitstring (tuple), LSB on right: 0x05 => 00000101
		   ** 2 - Polynomial form. 0x48 -> 0+x3+x6
		   ** | 0x100 : output in hex instead of decimal (for 0 and 2).
		   */
  char *str        /* Returns the tuple as a 0/1 string.
		    * Length GFSYMBOLSIZE+1 including trailing '\0'.*/
)
;
 char *GaloisPolyToString(  /* Convert polynomial to printable string. */
  GaloisField2 *GF,
  int ord,         /* Order of the polynomial. */
  GFSYMBOL *gpoly, /* Polynomial: Array of length ord+1. */
  char *polstr     /* String representation. Max length approx 10*(order+1).*/
)
;
