/* $Id: RStest.c,v 0.1 2004/09/20 20:35:35 jrothwei Exp $ */

/* Copyright (C) 2004  Sensicomm LLC
** Joseph Rothweiler http://www.sensicomm.com
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/* Simple test program for the Reed-Solomon encoder/decoder. */
/* Designed to run on an embedded system, where stdio may not be available. */

#define DOENCODE 1
#define DODECODE 1

// #define USESTDIO 1

#ifdef USESTDIO
#include <stdio.h>
// #include <unistd.h>
// #include <stdlib.h>
#endif

#include "GaloisMath.h"
#include "ReedSolomon.h"

#include "timingcheck.h"
#include "simpleio.h"

char *ProgName="rstest";

#define C2(a,b) ((a)<<8|(b))

TIMINGCHECK extimes[8];      /* Space for 8 clock calls. */

static GFSYMBOL encdata[256];
static GFSYMBOL rcvdata[256];
static GFSYMBOL outdata[256];

int main(int argc, char *argv[]) {
	GaloisField2 GF;
	int m = 8;   /* The field order. */
	int t = 7;   /* Number of correctable bits. 1 < t < 2**m/2 */
	GFSYMBOL g[2*t+1];  /* Tables for RS. */
	int datalen;
	int i;
	int er0, er1, eu0, eu1;
	GFSYMBOL *indata;

	say("start\n");

	/***********************************
	** First, Initialize the RS tables.
	************************************/

	/* The Galois field. */
	timingcheck(extimes+0);
	GaloisFieldAllocate(&GF,m);
#ifdef USESTDIO
	fprintf(stdout,"GF2 order %d size %d poly 0x%lx\n",GF.order,GF.size,GF.minpoly);
#endif
	GaloisFieldBuild(&GF);
	say("Initted\n");

	/* The RS tables. */
	timingcheck(extimes+1);
	RSGeneratorPoly(&GF,t,g);
	timingcheck(extimes+2);

	/***********************************
	** Encode some test data. 
	************************************/

#ifdef DOENCODE
	/* Generate arbitrary random data. */
	datalen=GF.size-2*t;
	indata=encdata+2*t;
	for(i=0;i<datalen;i++) indata[i] = 0;
	timingcheck(extimes+3);
	for(i=0;i<DOENCODE;i++) RS_encode(&GF,t,g,indata,encdata); 
	timingcheck(extimes+4);
	say("Encoded\n");
#endif

	/* Add a few errors. */

#ifdef DODECODE
	for(i=0;i<GF.size-1;i++) rcvdata[i] = encdata[i];
	rcvdata[7]  ^= 0xff;
	rcvdata[12] ^= 0x33;
	rcvdata[22] ^= 0x03;

	timingcheck(extimes+5);
	for(i=0;i<DODECODE;i++) RS_decode(&GF,t,rcvdata,outdata);
	timingcheck(extimes+6);
	say("Decoded\n");

	/* Summarize the results. */
	er0=er1=eu0=eu1 = 0;
	for(i=0;i<GF.size-1;i++) {
		if(encdata[i] != rcvdata[i]) {
			if(i<=2*t) er0++;
			else       er1++;
		}
		if(encdata[i] != outdata[i]) {
			if(i<=2*t) eu0++;
			else       eu1++;
		}
	}
	say("er0 "); sayi((unsigned)er0); say("\n");
	say("er1 "); sayi((unsigned)er1); say("\n");
	say("eu0 "); sayi((unsigned)eu0); say("\n");
	say("eu1 "); sayi((unsigned)eu1); say("\n");
	
#endif

	say("Done\n");
	for(i=0;i<7;i++) {
		sayi(i); say(" ");
		say(timingstring(extimes+i));
		say("\n");
	}
	say("Encode ");
	say(timingdeltastring(extimes+3,extimes+4));
	say("\n");
	say("Decode ");
	say(timingdeltastring(extimes+5,extimes+6));
	say("\n");
#ifdef USESTDIO
	for(i=0;i<GF.size-1;i++) {
		fprintf(stdout,"%3d: %c %5lx %5lx %5lx",
			i,(i<2*t)?'p':'.',
			(unsigned long)(encdata[i]),
			(unsigned long)(rcvdata[i]),
			(unsigned long)(outdata[i]));
		if(encdata[i] != rcvdata[i]) fprintf(stdout," %5lx",
			(unsigned long)(encdata[i]^rcvdata[i]));
		else                         fprintf(stdout," %5s",".");
		if(encdata[i] != outdata[i]) fprintf(stdout," %5lx\n",
				(unsigned long)(encdata[i]^outdata[i]));
		else                         fprintf(stdout," %5s\n",".");
	}
#endif
	return 0;
}
