/* $Id: simpleio.c,v 0.1 2004/09/20 20:35:35 jrothwei Exp $ */

/* Copyright (C) 2004  Sensicomm LLC
** Joseph Rothweiler http://www.sensicomm.com
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/* Routines to perform simple i/o operations when the full printf-style
** functionality is not needed. Currently, these routines run on
** a Linux-type platform,
** Analog Devices Blackfin processor with console i/o supported by the
** BF-533 EZ-KIT,
** Blackfin with i/o via the serial port.
*/

/* For Blackfin, define this to use the puts subroutine, **
** assuming the development system supports console io.  **
** Else, send output to the serial port at 38400 bps.    */
#define USECONSOLE 1

#if defined(__linux__)
#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#elif defined(__ADSPBLACKFIN__)
#include "misclib.h"
#ifdef USECONSOLE
#include <stdio.h>
#else
#include "bf533.h"
#include "bf533_uart.h"
#endif
#endif

/*Hstart*/ void simpleio_init(void)
/*Hend;*/ {
#ifndef USECONSOLE
	bf533_uart_configure(38400);
#endif
}
/*Hstart*/ void say(char *msg)
/*Hend;*/ {
#if defined(__linux__)
	printf("%s",msg);
	fflush(stdout);
#elif defined(__ADSPBLACKFIN__)
#ifdef USECONSOLE
	fputs(msg,stdout);
#else
	bf533_uart_sendstring(msg);
#endif
#else
#error No known platform defined.
#endif
}
/*Hstart*/ void sayi(unsigned val)
/*Hend;*/ {
#if defined(__linux__)
	printf("%u",val);
	fflush(stdout);
#elif defined(__ADSPBLACKFIN__)
	say(unsigned2staticstr(val));
#endif
}
/*Hstart*/ void sayx(unsigned val)
/*Hend;*/ {
#if defined(__linux__)
	printf("%x",val);
	fflush(stdout);
#elif defined(__ADSPBLACKFIN__)
	say((unsigned char *)long2staticstrhex((long)val));
#endif
}

