/* $Id: timingcheck.c,v 0.1 2004/09/20 20:35:35 jrothwei Exp $ */

/* Copyright (C) 2004  Sensicomm LLC
** Joseph Rothweiler http://www.sensicomm.com
** 
** This program is free software; you can redistribute it and/or
** modify it under the terms of the GNU General Public License
** as published by the Free Software Foundation; either version 2
** of the License, or (at your option) any later version.
** 
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
** 
** You should have received a copy of the GNU General Public License
** along with this program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/* Generic subroutines to obtain relative timing values from internal
** counters or system clocks. Values and differential values can be
** converted to strings in a system-dependent format.
*/

#if defined(__linux__)
#include <stdio.h>
#include <stdlib.h>
#endif

/*Hstart*/
#if defined(__linux__)
#include <unistd.h>
#include <sys/times.h>
#elif defined(__ADSPBLACKFIN__)
#include "misclib.h"
#endif
typedef struct {
#if defined(__linux__)
	struct tms tmssav;
	clock_t tmsclk;
#elif defined(__ADSPBLACKFIN__)
	unsigned int tm[2];
#else
	unsigned long tm[2];
#endif
} TIMINGCHECK
/*Hend;*/ ;
/*Hstart*/ void timingcheck(TIMINGCHECK *tt)
/*Hend;*/ {
#if defined(__linux__)
	tt->tmsclk = times(&(tt->tmssav));
#elif defined(__ADSPBLACKFIN__)
	bf533_cyctimes_raw(tt->tm);
#else
	tt->tm[0]=0;
	tt->tm[1]=0;
#endif
}
/*Hstart*/ char *timingstring(TIMINGCHECK *tt)
/*Hend;*/ {
#if defined(__linux__)
	static char strng[128];
	long ticks;
	ticks=sysconf(_SC_CLK_TCK);
	sprintf(strng," %ld clk %f usr %f sys %f usrc %f sysc %f",
		ticks,tt->tmsclk/(float)ticks,
		tt->tmssav.tms_utime/(float)ticks,
		tt->tmssav.tms_stime/(float)ticks,
		tt->tmssav.tms_cutime/(float)ticks,
		tt->tmssav.tms_cstime/(float)ticks );
	return strng;
#elif defined(__ADSPBLACKFIN__)
	static char str[18];
	long2strhex((long)(tt->tm[0]),str);
	str[8]=' ';
	long2strhex((long)(tt->tm[1]),str+9);
	return(str);
#else
	static char dmy[]="No clock";
	return dmy;
#endif
}
/*Hstart*/ char *timingdeltastring(TIMINGCHECK *tts,TIMINGCHECK *tte)
/*Hend;*/ {
#if defined(__linux__)
	static char strng[128];
	long ticks;
	ticks=sysconf(_SC_CLK_TCK);
	sprintf(strng," %ld clk %f usr %f sys %f usrc %f sysc %f",
		ticks,
		(tte->tmsclk - tts->tmsclk)/(float)ticks,
		(tte->tmssav.tms_utime  - tts->tmssav.tms_utime)/(float)ticks,
		(tte->tmssav.tms_stime  - tts->tmssav.tms_stime)/(float)ticks,
		(tte->tmssav.tms_cutime - tts->tmssav.tms_cutime)/(float)ticks,
		(tte->tmssav.tms_cstime - tts->tmssav.tms_cstime)/(float)ticks );
	return strng;
#elif defined(__ADSPBLACKFIN__)
	/* Really need a double-precision subtract here.       **
	** This will work as-is for short time intervals only. */
	static char str[18];
	long2strhex((long)(tte->tm[0]-tts->tm[0]),str);
	str[8]=' ';
	long2strhex((long)(tte->tm[1]-tts->tm[1]),str+9);
	return str;	
#else
	static char dmy[]="No clock";
	return dmy;
#endif
}
